<?php

/*
		Gestion de cartes
		
        Class for table : category.

        10/13/2022 12:52

        monmail@fai.com - 2022
        
        Licence libre etc ...

        Usage examples :

        <?php

            include_once("classdir/dbconnect.php);
            include_once("classdir/class_category.php);

            // Class instance
            $mycategory = category( $db );

            // Select all reccords :
            $results = $mycategory->selectAll();
            foreach($results as $row) {
                print_r($row);
            } 

            // Select reccord #12 :
            $id = 12;
            $row = $mycategory->select($id);
            print_r($row);

        ?>

*/
class _category {
      
    public $db;
    public $lasterror="";
    public $lastInsertId=-1;
    public $count = 0;
  
    /*
        Initialise object with dbHanler.
                
    */    
    function __construct( $db ) {
        $this->db = $db;
    }

    /*
        Select one reccord from category.
        
        parameters : $id (reccord id).
        return : associative array of reccord.
    */
    function select($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.valid as 'a.valid',
			a.checked as 'a.checked',
			a.externaldata as 'a.externaldata',
			a.form as 'a.form',
			a.iconurl as 'a.iconurl',
			a.owner as 'a.owner',
			a.map_id as 'a.map_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name' 

		from category a 
 			left outer join map b1 on b1.id =  a.map_id 

		where a.id = :id; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    function selectPrev($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.valid as 'a.valid',
			a.checked as 'a.checked',
			a.externaldata as 'a.externaldata',
			a.form as 'a.form',
			a.iconurl as 'a.iconurl',
			a.owner as 'a.owner',
			a.map_id as 'a.map_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name' 

		from category a 
 			left outer join map b1 on b1.id =  a.map_id 

		where a.id =  (SELECT id FROM category WHERE id < :id ORDER BY id DESC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }


    function selectNext($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.valid as 'a.valid',
			a.checked as 'a.checked',
			a.externaldata as 'a.externaldata',
			a.form as 'a.form',
			a.iconurl as 'a.iconurl',
			a.owner as 'a.owner',
			a.map_id as 'a.map_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name' 

		from category a 
 			left outer join map b1 on b1.id =  a.map_id 

		where a.id =  (SELECT id FROM category WHERE id > :id ORDER BY id ASC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    /*
        Select all reccords from category  (with default order).
        
        parameters : None
        return : associative array of reccords.
    */
    function selectAll() {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.valid as 'a.valid',
			a.checked as 'a.checked',
			a.externaldata as 'a.externaldata',
			a.form as 'a.form',
			a.iconurl as 'a.iconurl',
			a.owner as 'a.owner',
			a.map_id as 'a.map_id',

 			b1.id as 'b1.id',
 			b1.name as 'b1.name'

		from category a 
 			left outer join map b1 on b1.id =  a.map_id 

		;    
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);           
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            $this->count = 0;
            return false;
        }
    }

    /*
        Update category 
        
        parameters : $id (reccord id) and fields list values.
        return : True if no error.
    */
    function update( $id, $name,$valid,$checked,$externaldata,$form,$iconurl,$owner,$map_id ) {

        $this->lasterror="";        

		if ($name == '') $name = null;
		if ($valid == '') $valid = null;
		if ($checked == '') $checked = null;
		if ($externaldata == '') $externaldata = null;
		if ($form == '') $form = null;
		if ($iconurl == '') $iconurl = null;
		if ($owner == '') $owner = null;
		if ($map_id == '') $map_id = null;
		

        if ($this->checkValue( array( 
		$name => '',
		$valid => '',
		$checked => '',
		$externaldata => '',
		$form => '',
		$iconurl => '',
		$owner => '',
		$map_id => ''

) ) == false )
        {
            $this->lasterror="Check fields false";        
            return false;
        }

        $sqlSt="
		update category set
			name = :name,
			valid = :valid,
			checked = :checked,
			externaldata = :externaldata,
			form = :form,
			iconurl = :iconurl,
			owner = :owner,
			map_id = :map_id

		where id = :id; 
            ";

        try {        
            
            $stmt = $this->db->prepare($sqlSt);
            
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);

			$stmt->bindParam(':name', $name, PDO::PARAM_STR); 
			$stmt->bindParam(':valid', $valid, PDO::PARAM_INT); 
			$stmt->bindParam(':checked', $checked, PDO::PARAM_INT); 
			$stmt->bindParam(':externaldata', $externaldata, PDO::PARAM_INT); 
			$stmt->bindParam(':form', $form, PDO::PARAM_STR); 
			$stmt->bindParam(':iconurl', $iconurl, PDO::PARAM_STR); 
			$stmt->bindParam(':owner', $owner, PDO::PARAM_STR); 
			$stmt->bindParam(':map_id', $map_id, PDO::PARAM_INT); 
 
            $stmt->execute();             
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql update '.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>valid</b> = [$valid],<b>checked</b> = [$checked],<b>externaldata</b> = [$externaldata],<b>form</b> = [$form],<b>iconurl</b> = [$iconurl],<b>owner</b> = [$owner],<b>map_id</b> = [$map_id]";
				
				return false;
			}   
			         
            return true;

        } catch (Exception $e) {            
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>valid</b> = [$valid],<b>checked</b> = [$checked],<b>externaldata</b> = [$externaldata],<b>form</b> = [$form],<b>iconurl</b> = [$iconurl],<b>owner</b> = [$owner],<b>map_id</b> = [$map_id]";  
            return false;
        }

    }

    /*
        Insert new reccord into category.
        
        Parameters : fields values list.
        return : True if no error.
    */
    function insert( $name,$valid,$checked,$externaldata,$form,$iconurl,$owner,$map_id ) {

        $this->lasterror="";
        $this->lastInsertId = -1;

		if ($name == '') $name = null;
		if ($valid == '') $valid = null;
		if ($checked == '') $checked = null;
		if ($externaldata == '') $externaldata = null;
		if ($form == '') $form = null;
		if ($iconurl == '') $iconurl = null;
		if ($owner == '') $owner = null;
		if ($map_id == '') $map_id = null;
		
		
        if ($this->checkValue( array( 
		$name => '',
		$valid => '',
		$checked => '',
		$externaldata => '',
		$form => '',
		$iconurl => '',
		$owner => '',
		$map_id => ''

) ) == false )
        {
            $this->lasterror="Check fields error";        
            return false;
        }

        $sqlSt="
		insert into category (
		name,
		valid,
		checked,
		externaldata,
		form,
		iconurl,
		owner,
		map_id
 
			)
		values (
		:name, 
		:valid, 
		:checked, 
		:externaldata, 
		:form, 
		:iconurl, 
		:owner, 
		:map_id 
 
			);    
            ";

        try {        

            $stmt = $this->db->prepare($sqlSt);               
			
			$stmt->bindParam(':name', $name, PDO::PARAM_STR); 
			$stmt->bindParam(':valid', $valid, PDO::PARAM_INT); 
			$stmt->bindParam(':checked', $checked, PDO::PARAM_INT); 
			$stmt->bindParam(':externaldata', $externaldata, PDO::PARAM_INT); 
			$stmt->bindParam(':form', $form, PDO::PARAM_STR); 
			$stmt->bindParam(':iconurl', $iconurl, PDO::PARAM_STR); 
			$stmt->bindParam(':owner', $owner, PDO::PARAM_STR); 
			$stmt->bindParam(':map_id', $map_id, PDO::PARAM_INT); 
 

            $stmt->execute();     
            $this->lastInsertId = $this->db->lastInsertId(); 
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql insert'.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>valid</b> = [$valid],<b>checked</b> = [$checked],<b>externaldata</b> = [$externaldata],<b>form</b> = [$form],<b>iconurl</b> = [$iconurl],<b>owner</b> = [$owner],<b>map_id</b> = [$map_id]";
				return false;
			}              
                   
            return true;

        } catch (Exception $e) {
             $this->lasterror = $e->getMessage().'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>valid</b> = [$valid],<b>checked</b> = [$checked],<b>externaldata</b> = [$externaldata],<b>form</b> = [$form],<b>iconurl</b> = [$iconurl],<b>owner</b> = [$owner],<b>map_id</b> = [$map_id]";
            return false;
        }
    }

    /*
        Delete one reccord from category.
        
        parameters : $id (reccord id).
        return : True if no error.
    */
    function delete($id) {

        $this->lasterror="";

        $sqlSt="
		delete from category             
		where id = :id; 
            ";   
     
        try {        

            $stmt = $this->db->prepare($sqlSt);  
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute();        
            
			if ($stmt->rowCount() == 0) {
				$this->lasterror = "Erreur Sql delete ".' (<b>Id</b> ='.$id.')';       
				return false;
			}              

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b>='.$id.')';
            return false;
        }
        return true;
    }

    /*
        Get id min, id max, record count from category  
        
        parameters : None
        return : count value or False in case of error
    */
	function getCount() {
		
        $this->lasterror="";

        $sqlSt="
		select min(id) as minid, max(id) as maxid, count(*) as count 
		from category 
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC); 
            return $results[0];          

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            return array( 'minid' => -1, 'maxid' => -1, 'count' => -1);
        }
        		
		//return $this->count;
		return $results[0];
	}




	public $colmeta = [
	'a.id' => array (
		'metades' => "IDentifiant",
		'sortformat' => "{{sortformat}}",
		'listlen' => 4,
		'listlib' => 'Id',
		'listlink' => 'on',
		'selectlen' => 4,
		'selectlib' => 'Id',
		'selectlink' => 'on',
		'selectdisplay' => '',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => '', 
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 

		),
	'a.name' => array ( 
		'metades' => "Nom de la couche (layer)",
		'sortformat' => "", 
		'listlen' => 32,
		'listlib' => 'Nom' , 
		'listlink' => '',
		'selectlen' => 32,
		'selectlib' => 'Nom' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.valid' => array ( 
		'metades' => "Visible sur la page publique",
		'sortformat' => "", 
		'listlen' => 3,
		'listlib' => 'Visible' , 
		'listlink' => '',
		'selectlen' => 3,
		'selectlib' => 'Visible' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.checked' => array ( 
		'metades' => "Couche activée par défaut",
		'sortformat' => "", 
		'listlen' => 3,
		'listlib' => 'Activée' , 
		'listlink' => '',
		'selectlen' => 0,
		'selectlib' => 'Activée' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.externaldata' => array ( 
		'metades' => "Données externes à la base de données",
		'sortformat' => "", 
		'listlen' => 3,
		'listlib' => 'Externe' , 
		'listlink' => '',
		'selectlen' => 3,
		'selectlib' => 'Externe' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.form' => array ( 
		'metades' => "Identifiant du formulaire des données",
		'sortformat' => "", 
		'listlen' => 12,
		'listlib' => 'Formulaire' , 
		'listlink' => '',
		'selectlen' => 0,
		'selectlib' => 'Formulaire' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.iconurl' => array ( 
		'metades' => "Url  par défaut de lʼicône des markers",
		'sortformat' => "", 
		'listlen' => 20,
		'listlib' => 'Url Icône' , 
		'listlink' => '',
		'selectlen' => 20,
		'selectlib' => 'Url Icône' , 
		'selectlink' => '',
		'selectdisplay' => 'icondoc',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'urlpic',
		'listdisplaywidth' => '50', 
		'listdisplayheight' => '50' 
),
	'a.owner' => array ( 
		'metades' => "Propriétaire",
		'sortformat' => "", 
		'listlen' => 12,
		'listlib' => 'Propriétaire' , 
		'listlink' => '',
		'selectlen' => 0,
		'selectlib' => 'Propriétaire' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.map_id' => array ( 
		'metades' => "Carte",
		'sortformat' => "", 
		'listlen' => 0,
		'listlib' => 'Carte' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'Carte' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),

 	'b1.id' => array (  // Map
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'id', 
		'listlink' => '' ,
		'selectlen' => 0,
		'selectlib' => 'id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.name' => array (  // Map
		'metades' => "",
		'sortformat' => "",
		'listlen' => 20,
		'listlib' => 'name', 
		'listlink' => '' ,
		'selectlen' => 25,
		'selectlib' => 'name', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
) 
			
	];

    /*
        Get field property value
        
        Parameters : field name and property name.
        return : value of property.
    */

	function getMeta( $fieldkey , $metakey ) {
		return $this->colmeta [$fieldkey][$metakey];
	}

    /*
        Get labels for list table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderList() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['listlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['listlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenList( $field ) {		
		
		return $this->colmeta[$field]['listlen'];
	}

	function getTableFieldLinkList( $field ) {		
		
		return $this->colmeta[$field]['listlink'];
	}

    /*
        Get labels for select table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderSelect() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['selectlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['selectlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlen'];
	}

	function getTableFieldLinkSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlink'];
	}

	
    /*
        Check user input values for category.
        
        Parameters : array of fields with regexp  paterns. ex: array( $nom =>'^(?!\s*$).+' , $icone => '')
        return : False if at last one regexp does not match.
    */
    function checkValue( $arrayCheck ) {

        $indexfield=1;
        foreach ( $arrayCheck as $value => $pattern) {

            if ($pattern=='') continue;

            $pattern='/'.$pattern.'/';

            if (!preg_match($pattern, $value)) {
                $this->lasterror = "Erreur champ #".$indexfield. '. Valeur:['.$value.']';
                return false;                
            }

            $indexfield+=1;

        }

        return true;
    }

} // end class

?>
