<?php

/*
		Gestion de cartes
		
        Class for table : propertylib.

        10/13/2022 12:52

        monmail@fai.com - 2022
        
        Licence libre etc ...

        Usage examples :

        <?php

            include_once("classdir/dbconnect.php);
            include_once("classdir/class_propertylib.php);

            // Class instance
            $mypropertylib = propertylib( $db );

            // Select all reccords :
            $results = $mypropertylib->selectAll();
            foreach($results as $row) {
                print_r($row);
            } 

            // Select reccord #12 :
            $id = 12;
            $row = $mypropertylib->select($id);
            print_r($row);

        ?>

*/
class _propertylib {
      
    public $db;
    public $lasterror="";
    public $lastInsertId=-1;
    public $count = 0;
  
    /*
        Initialise object with dbHanler.
                
    */    
    function __construct( $db ) {
        $this->db = $db;
    }

    /*
        Select one reccord from propertylib.
        
        parameters : $id (reccord id).
        return : associative array of reccord.
    */
    function select($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.propertyref_id as 'a.propertyref_id',
			a.libid as 'a.libid',
			a.Lib as 'a.Lib',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.description as 'b1.description', 
 			b1.category_id as 'b1.category_id' 

		from propertylib a 
 			left outer join propertyref b1 on b1.id =  a.propertyref_id 

		where a.id = :id; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    function selectPrev($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.propertyref_id as 'a.propertyref_id',
			a.libid as 'a.libid',
			a.Lib as 'a.Lib',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.description as 'b1.description', 
 			b1.category_id as 'b1.category_id' 

		from propertylib a 
 			left outer join propertyref b1 on b1.id =  a.propertyref_id 

		where a.id =  (SELECT id FROM propertylib WHERE id < :id ORDER BY id DESC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }


    function selectNext($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.propertyref_id as 'a.propertyref_id',
			a.libid as 'a.libid',
			a.Lib as 'a.Lib',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.description as 'b1.description', 
 			b1.category_id as 'b1.category_id' 

		from propertylib a 
 			left outer join propertyref b1 on b1.id =  a.propertyref_id 

		where a.id =  (SELECT id FROM propertylib WHERE id > :id ORDER BY id ASC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    /*
        Select all reccords from propertylib  (with default order).
        
        parameters : None
        return : associative array of reccords.
    */
    function selectAll() {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.propertyref_id as 'a.propertyref_id',
			a.libid as 'a.libid',
			a.Lib as 'a.Lib',

 			b1.id as 'b1.id',
 			b1.name as 'b1.name',
 			b1.description as 'b1.description',
 			b1.category_id as 'b1.category_id'

		from propertylib a 
 			left outer join propertyref b1 on b1.id =  a.propertyref_id 

		;    
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);           
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            $this->count = 0;
            return false;
        }
    }

    /*
        Update propertylib 
        
        parameters : $id (reccord id) and fields list values.
        return : True if no error.
    */
    function update( $id, $propertyref_id,$libid,$Lib ) {

        $this->lasterror="";        

		if ($propertyref_id == '') $propertyref_id = null;
		if ($libid == '') $libid = null;
		if ($Lib == '') $Lib = null;
		

        if ($this->checkValue( array( 
		$propertyref_id => '',
		$libid => '',
		$Lib => ''

) ) == false )
        {
            $this->lasterror="Check fields false";        
            return false;
        }

        $sqlSt="
		update propertylib set
			propertyref_id = :propertyref_id,
			libid = :libid,
			Lib = :Lib

		where id = :id; 
            ";

        try {        
            
            $stmt = $this->db->prepare($sqlSt);
            
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);

			$stmt->bindParam(':propertyref_id', $propertyref_id, PDO::PARAM_INT); 
			$stmt->bindParam(':libid', $libid, PDO::PARAM_INT); 
			$stmt->bindParam(':Lib', $Lib, PDO::PARAM_STR); 
 
            $stmt->execute();             
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql update '.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>propertyref_id</b> = [$propertyref_id],<b>libid</b> = [$libid],<b>Lib</b> = [$Lib]";
				
				return false;
			}   
			         
            return true;

        } catch (Exception $e) {            
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>propertyref_id</b> = [$propertyref_id],<b>libid</b> = [$libid],<b>Lib</b> = [$Lib]";  
            return false;
        }

    }

    /*
        Insert new reccord into propertylib.
        
        Parameters : fields values list.
        return : True if no error.
    */
    function insert( $propertyref_id,$libid,$Lib ) {

        $this->lasterror="";
        $this->lastInsertId = -1;

		if ($propertyref_id == '') $propertyref_id = null;
		if ($libid == '') $libid = null;
		if ($Lib == '') $Lib = null;
		
		
        if ($this->checkValue( array( 
		$propertyref_id => '',
		$libid => '',
		$Lib => ''

) ) == false )
        {
            $this->lasterror="Check fields error";        
            return false;
        }

        $sqlSt="
		insert into propertylib (
		propertyref_id,
		libid,
		Lib
 
			)
		values (
		:propertyref_id, 
		:libid, 
		:Lib 
 
			);    
            ";

        try {        

            $stmt = $this->db->prepare($sqlSt);               
			
			$stmt->bindParam(':propertyref_id', $propertyref_id, PDO::PARAM_INT); 
			$stmt->bindParam(':libid', $libid, PDO::PARAM_INT); 
			$stmt->bindParam(':Lib', $Lib, PDO::PARAM_STR); 
 

            $stmt->execute();     
            $this->lastInsertId = $this->db->lastInsertId(); 
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql insert'.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>propertyref_id</b> = [$propertyref_id],<b>libid</b> = [$libid],<b>Lib</b> = [$Lib]";
				return false;
			}              
                   
            return true;

        } catch (Exception $e) {
             $this->lasterror = $e->getMessage().'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>propertyref_id</b> = [$propertyref_id],<b>libid</b> = [$libid],<b>Lib</b> = [$Lib]";
            return false;
        }
    }

    /*
        Delete one reccord from propertylib.
        
        parameters : $id (reccord id).
        return : True if no error.
    */
    function delete($id) {

        $this->lasterror="";

        $sqlSt="
		delete from propertylib             
		where id = :id; 
            ";   
     
        try {        

            $stmt = $this->db->prepare($sqlSt);  
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute();        
            
			if ($stmt->rowCount() == 0) {
				$this->lasterror = "Erreur Sql delete ".' (<b>Id</b> ='.$id.')';       
				return false;
			}              

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b>='.$id.')';
            return false;
        }
        return true;
    }

    /*
        Get id min, id max, record count from propertylib  
        
        parameters : None
        return : count value or False in case of error
    */
	function getCount() {
		
        $this->lasterror="";

        $sqlSt="
		select min(id) as minid, max(id) as maxid, count(*) as count 
		from propertylib 
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC); 
            return $results[0];          

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            return array( 'minid' => -1, 'maxid' => -1, 'count' => -1);
        }
        		
		//return $this->count;
		return $results[0];
	}




	public $colmeta = [
	'a.id' => array (
		'metades' => "",
		'sortformat' => "{{sortformat}}",
		'listlen' => 4,
		'listlib' => 'a_id',
		'listlink' => 'on',
		'selectlen' => 4,
		'selectlib' => 'a_id',
		'selectlink' => '',
		'selectdisplay' => '',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => '', 
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 

		),
	'a.propertyref_id' => array ( 
		'metades' => "",
		'sortformat' => "", 
		'listlen' => 4,
		'listlib' => 'a_propertyref_id' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'a_propertyref_id' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.libid' => array ( 
		'metades' => "",
		'sortformat' => "", 
		'listlen' => 4,
		'listlib' => 'a_libid' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'a_libid' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.Lib' => array ( 
		'metades' => "",
		'sortformat' => "", 
		'listlen' => 12,
		'listlib' => 'a_Lib' , 
		'listlink' => '',
		'selectlen' => 12,
		'selectlib' => 'a_Lib' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),

 	'b1.id' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.name' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'name', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'name', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.description' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'description', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'description', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.category_id' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'category_id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'category_id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
) 
			
	];

    /*
        Get field property value
        
        Parameters : field name and property name.
        return : value of property.
    */

	function getMeta( $fieldkey , $metakey ) {
		return $this->colmeta [$fieldkey][$metakey];
	}

    /*
        Get labels for list table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderList() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['listlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['listlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenList( $field ) {		
		
		return $this->colmeta[$field]['listlen'];
	}

	function getTableFieldLinkList( $field ) {		
		
		return $this->colmeta[$field]['listlink'];
	}

    /*
        Get labels for select table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderSelect() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['selectlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['selectlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlen'];
	}

	function getTableFieldLinkSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlink'];
	}

	
    /*
        Check user input values for propertylib.
        
        Parameters : array of fields with regexp  paterns. ex: array( $nom =>'^(?!\s*$).+' , $icone => '')
        return : False if at last one regexp does not match.
    */
    function checkValue( $arrayCheck ) {

        $indexfield=1;
        foreach ( $arrayCheck as $value => $pattern) {

            if ($pattern=='') continue;

            $pattern='/'.$pattern.'/';

            if (!preg_match($pattern, $value)) {
                $this->lasterror = "Erreur champ #".$indexfield. '. Valeur:['.$value.']';
                return false;                
            }

            $indexfield+=1;

        }

        return true;
    }

} // end class

?>
