<?php

/*
		Gestion de cartes
		
        Class for table : propertyref.

        10/13/2022 12:52

        monmail@fai.com - 2022
        
        Licence libre etc ...

        Usage examples :

        <?php

            include_once("classdir/dbconnect.php);
            include_once("classdir/class_propertyref.php);

            // Class instance
            $mypropertyref = propertyref( $db );

            // Select all reccords :
            $results = $mypropertyref->selectAll();
            foreach($results as $row) {
                print_r($row);
            } 

            // Select reccord #12 :
            $id = 12;
            $row = $mypropertyref->select($id);
            print_r($row);

        ?>

*/
class _propertyref {
      
    public $db;
    public $lasterror="";
    public $lastInsertId=-1;
    public $count = 0;
  
    /*
        Initialise object with dbHanler.
                
    */    
    function __construct( $db ) {
        $this->db = $db;
    }

    /*
        Select one reccord from propertyref.
        
        parameters : $id (reccord id).
        return : associative array of reccord.
    */
    function select($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.description as 'a.description',
			a.category_id as 'a.category_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.valid as 'b1.valid', 
 			b1.checked as 'b1.checked', 
 			b1.externaldata as 'b1.externaldata', 
 			b1.form as 'b1.form', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.owner as 'b1.owner', 
 			b1.map_id as 'b1.map_id' 

		from propertyref a 
 			left outer join category b1 on b1.id =  a.category_id 

		where a.id = :id; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    function selectPrev($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.description as 'a.description',
			a.category_id as 'a.category_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.valid as 'b1.valid', 
 			b1.checked as 'b1.checked', 
 			b1.externaldata as 'b1.externaldata', 
 			b1.form as 'b1.form', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.owner as 'b1.owner', 
 			b1.map_id as 'b1.map_id' 

		from propertyref a 
 			left outer join category b1 on b1.id =  a.category_id 

		where a.id =  (SELECT id FROM propertyref WHERE id < :id ORDER BY id DESC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }


    function selectNext($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.description as 'a.description',
			a.category_id as 'a.category_id',

 			b1.id as 'b1.id', 
 			b1.name as 'b1.name', 
 			b1.valid as 'b1.valid', 
 			b1.checked as 'b1.checked', 
 			b1.externaldata as 'b1.externaldata', 
 			b1.form as 'b1.form', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.owner as 'b1.owner', 
 			b1.map_id as 'b1.map_id' 

		from propertyref a 
 			left outer join category b1 on b1.id =  a.category_id 

		where a.id =  (SELECT id FROM propertyref WHERE id > :id ORDER BY id ASC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    /*
        Select all reccords from propertyref  (with default order).
        
        parameters : None
        return : associative array of reccords.
    */
    function selectAll() {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.name as 'a.name',
			a.description as 'a.description',
			a.category_id as 'a.category_id',

 			b1.id as 'b1.id',
 			b1.name as 'b1.name',
 			b1.valid as 'b1.valid',
 			b1.checked as 'b1.checked',
 			b1.externaldata as 'b1.externaldata',
 			b1.form as 'b1.form',
 			b1.iconurl as 'b1.iconurl',
 			b1.owner as 'b1.owner',
 			b1.map_id as 'b1.map_id'

		from propertyref a 
 			left outer join category b1 on b1.id =  a.category_id 

		;    
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);           
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            $this->count = 0;
            return false;
        }
    }

    /*
        Update propertyref 
        
        parameters : $id (reccord id) and fields list values.
        return : True if no error.
    */
    function update( $id, $name,$description,$category_id ) {

        $this->lasterror="";        

		if ($name == '') $name = null;
		if ($description == '') $description = null;
		if ($category_id == '') $category_id = null;
		

        if ($this->checkValue( array( 
		$name => '',
		$description => '',
		$category_id => ''

) ) == false )
        {
            $this->lasterror="Check fields false";        
            return false;
        }

        $sqlSt="
		update propertyref set
			name = :name,
			description = :description,
			category_id = :category_id

		where id = :id; 
            ";

        try {        
            
            $stmt = $this->db->prepare($sqlSt);
            
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);

			$stmt->bindParam(':name', $name, PDO::PARAM_STR); 
			$stmt->bindParam(':description', $description, PDO::PARAM_STR); 
			$stmt->bindParam(':category_id', $category_id, PDO::PARAM_INT); 
 
            $stmt->execute();             
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql update '.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>description</b> = [$description],<b>category_id</b> = [$category_id]";
				
				return false;
			}   
			         
            return true;

        } catch (Exception $e) {            
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>description</b> = [$description],<b>category_id</b> = [$category_id]";  
            return false;
        }

    }

    /*
        Insert new reccord into propertyref.
        
        Parameters : fields values list.
        return : True if no error.
    */
    function insert( $name,$description,$category_id ) {

        $this->lasterror="";
        $this->lastInsertId = -1;

		if ($name == '') $name = null;
		if ($description == '') $description = null;
		if ($category_id == '') $category_id = null;
		
		
        if ($this->checkValue( array( 
		$name => '',
		$description => '',
		$category_id => ''

) ) == false )
        {
            $this->lasterror="Check fields error";        
            return false;
        }

        $sqlSt="
		insert into propertyref (
		name,
		description,
		category_id
 
			)
		values (
		:name, 
		:description, 
		:category_id 
 
			);    
            ";

        try {        

            $stmt = $this->db->prepare($sqlSt);               
			
			$stmt->bindParam(':name', $name, PDO::PARAM_STR); 
			$stmt->bindParam(':description', $description, PDO::PARAM_STR); 
			$stmt->bindParam(':category_id', $category_id, PDO::PARAM_INT); 
 

            $stmt->execute();     
            $this->lastInsertId = $this->db->lastInsertId(); 
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql insert'.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>description</b> = [$description],<b>category_id</b> = [$category_id]";
				return false;
			}              
                   
            return true;

        } catch (Exception $e) {
             $this->lasterror = $e->getMessage().'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>name</b> = [$name],<b>description</b> = [$description],<b>category_id</b> = [$category_id]";
            return false;
        }
    }

    /*
        Delete one reccord from propertyref.
        
        parameters : $id (reccord id).
        return : True if no error.
    */
    function delete($id) {

        $this->lasterror="";

        $sqlSt="
		delete from propertyref             
		where id = :id; 
            ";   
     
        try {        

            $stmt = $this->db->prepare($sqlSt);  
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute();        
            
			if ($stmt->rowCount() == 0) {
				$this->lasterror = "Erreur Sql delete ".' (<b>Id</b> ='.$id.')';       
				return false;
			}              

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b>='.$id.')';
            return false;
        }
        return true;
    }

    /*
        Get id min, id max, record count from propertyref  
        
        parameters : None
        return : count value or False in case of error
    */
	function getCount() {
		
        $this->lasterror="";

        $sqlSt="
		select min(id) as minid, max(id) as maxid, count(*) as count 
		from propertyref 
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC); 
            return $results[0];          

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            return array( 'minid' => -1, 'maxid' => -1, 'count' => -1);
        }
        		
		//return $this->count;
		return $results[0];
	}




	public $colmeta = [
	'a.id' => array (
		'metades' => "IDentifiant",
		'sortformat' => "{{sortformat}}",
		'listlen' => 4,
		'listlib' => 'Id',
		'listlink' => 'on',
		'selectlen' => 4,
		'selectlib' => 'Id',
		'selectlink' => 'on',
		'selectdisplay' => '',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => '', 
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 

		),
	'a.name' => array ( 
		'metades' => "Nom de la propriété",
		'sortformat' => "", 
		'listlen' => 12,
		'listlib' => 'Nom' , 
		'listlink' => '',
		'selectlen' => 12,
		'selectlib' => 'Nom' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.description' => array ( 
		'metades' => "Description de la propriété",
		'sortformat' => "", 
		'listlen' => 44,
		'listlib' => 'Description' , 
		'listlink' => '',
		'selectlen' => 44,
		'selectlib' => 'Description' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.category_id' => array ( 
		'metades' => "Couche (layer) de la carte",
		'sortformat' => "", 
		'listlen' => 0,
		'listlib' => 'Couche' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'Couche' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),

 	'b1.id' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.name' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'name', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'name', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.valid' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'valid', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'valid', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.checked' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'checked', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'checked', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.externaldata' => array (  // Category
		'metades' => "Données externes à la base de données",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'externaldata', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'externaldata', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.form' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'form', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'form', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.iconurl' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'iconurl', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'iconurl', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.owner' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'owner', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'owner', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.map_id' => array (  // Category
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'map_id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'map_id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
) 
			
	];

    /*
        Get field property value
        
        Parameters : field name and property name.
        return : value of property.
    */

	function getMeta( $fieldkey , $metakey ) {
		return $this->colmeta [$fieldkey][$metakey];
	}

    /*
        Get labels for list table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderList() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['listlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['listlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenList( $field ) {		
		
		return $this->colmeta[$field]['listlen'];
	}

	function getTableFieldLinkList( $field ) {		
		
		return $this->colmeta[$field]['listlink'];
	}

    /*
        Get labels for select table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderSelect() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['selectlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['selectlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlen'];
	}

	function getTableFieldLinkSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlink'];
	}

	
    /*
        Check user input values for propertyref.
        
        Parameters : array of fields with regexp  paterns. ex: array( $nom =>'^(?!\s*$).+' , $icone => '')
        return : False if at last one regexp does not match.
    */
    function checkValue( $arrayCheck ) {

        $indexfield=1;
        foreach ( $arrayCheck as $value => $pattern) {

            if ($pattern=='') continue;

            $pattern='/'.$pattern.'/';

            if (!preg_match($pattern, $value)) {
                $this->lasterror = "Erreur champ #".$indexfield. '. Valeur:['.$value.']';
                return false;                
            }

            $indexfield+=1;

        }

        return true;
    }

} // end class

?>
