<?php

/*
		Gestion de cartes
		
        Class for table : propertyval.

        10/13/2022 12:52

        monmail@fai.com - 2022
        
        Licence libre etc ...

        Usage examples :

        <?php

            include_once("classdir/dbconnect.php);
            include_once("classdir/class_propertyval.php);

            // Class instance
            $mypropertyval = propertyval( $db );

            // Select all reccords :
            $results = $mypropertyval->selectAll();
            foreach($results as $row) {
                print_r($row);
            } 

            // Select reccord #12 :
            $id = 12;
            $row = $mypropertyval->select($id);
            print_r($row);

        ?>

*/
class _propertyval {
      
    public $db;
    public $lasterror="";
    public $lastInsertId=-1;
    public $count = 0;
  
    /*
        Initialise object with dbHanler.
                
    */    
    function __construct( $db ) {
        $this->db = $db;
    }

    /*
        Select one reccord from propertyval.
        
        parameters : $id (reccord id).
        return : associative array of reccord.
    */
    function select($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.marker_id as 'a.marker_id',
			a.propertyref_id as 'a.propertyref_id',
			a.value as 'a.value',
			a.datecreated as 'a.datecreated',

 			b1.id as 'b1.id', 
 			b1.key as 'b1.key', 
 			b1.name as 'b1.name', 
 			b1.type as 'b1.type', 
 			b1.description as 'b1.description', 
 			b1.lat as 'b1.lat', 
 			b1.lon as 'b1.lon', 
 			b1.alt as 'b1.alt', 
 			b1.url as 'b1.url', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.photoprefix as 'b1.photoprefix', 
 			b1.datemaj as 'b1.datemaj', 
 			b1.category_id as 'b1.category_id', 
 			b1.datecreated as 'b1.datecreated', 
 			c2.id as 'c2.id', 
 			c2.name as 'c2.name', 
 			c2.description as 'c2.description', 
 			c2.category_id as 'c2.category_id' 

		from propertyval a 
 			left outer join propertyref c2 on c2.id =  a.propertyref_id 
 			left outer join marker b1 on b1.id =  a.marker_id 

		where a.id = :id; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    function selectPrev($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.marker_id as 'a.marker_id',
			a.propertyref_id as 'a.propertyref_id',
			a.value as 'a.value',
			a.datecreated as 'a.datecreated',

 			b1.id as 'b1.id', 
 			b1.key as 'b1.key', 
 			b1.name as 'b1.name', 
 			b1.type as 'b1.type', 
 			b1.description as 'b1.description', 
 			b1.lat as 'b1.lat', 
 			b1.lon as 'b1.lon', 
 			b1.alt as 'b1.alt', 
 			b1.url as 'b1.url', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.photoprefix as 'b1.photoprefix', 
 			b1.datemaj as 'b1.datemaj', 
 			b1.category_id as 'b1.category_id', 
 			b1.datecreated as 'b1.datecreated', 
 			c2.id as 'c2.id', 
 			c2.name as 'c2.name', 
 			c2.description as 'c2.description', 
 			c2.category_id as 'c2.category_id' 

		from propertyval a 
 			left outer join propertyref c2 on c2.id =  a.propertyref_id 
 			left outer join marker b1 on b1.id =  a.marker_id 

		where a.id =  (SELECT id FROM propertyval WHERE id < :id ORDER BY id DESC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }


    function selectNext($id) {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.marker_id as 'a.marker_id',
			a.propertyref_id as 'a.propertyref_id',
			a.value as 'a.value',
			a.datecreated as 'a.datecreated',

 			b1.id as 'b1.id', 
 			b1.key as 'b1.key', 
 			b1.name as 'b1.name', 
 			b1.type as 'b1.type', 
 			b1.description as 'b1.description', 
 			b1.lat as 'b1.lat', 
 			b1.lon as 'b1.lon', 
 			b1.alt as 'b1.alt', 
 			b1.url as 'b1.url', 
 			b1.iconurl as 'b1.iconurl', 
 			b1.photoprefix as 'b1.photoprefix', 
 			b1.datemaj as 'b1.datemaj', 
 			b1.category_id as 'b1.category_id', 
 			b1.datecreated as 'b1.datecreated', 
 			c2.id as 'c2.id', 
 			c2.name as 'c2.name', 
 			c2.description as 'c2.description', 
 			c2.category_id as 'c2.category_id' 

		from propertyval a 
 			left outer join propertyref c2 on c2.id =  a.propertyref_id 
 			left outer join marker b1 on b1.id =  a.marker_id 

		where a.id =  (SELECT id FROM propertyval WHERE id > :id ORDER BY id ASC LIMIT 1) ; 
			";

        try {        		

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$idkeyname.')';
            return false;
        }
    }

    /*
        Select all reccords from propertyval  (with default order).
        
        parameters : None
        return : associative array of reccords.
    */
    function selectAll() {

        $this->lasterror="";

        $sqlSt="
		select       
			a.id as 'a.id',  
			a.marker_id as 'a.marker_id',
			a.propertyref_id as 'a.propertyref_id',
			a.value as 'a.value',
			a.datecreated as 'a.datecreated',

 			b1.id as 'b1.id',
 			b1.key as 'b1.key',
 			b1.name as 'b1.name',
 			b1.type as 'b1.type',
 			b1.description as 'b1.description',
 			b1.lat as 'b1.lat',
 			b1.lon as 'b1.lon',
 			b1.alt as 'b1.alt',
 			b1.url as 'b1.url',
 			b1.iconurl as 'b1.iconurl',
 			b1.photoprefix as 'b1.photoprefix',
 			b1.datemaj as 'b1.datemaj',
 			b1.category_id as 'b1.category_id',
 			b1.datecreated as 'b1.datecreated',
 			c2.id as 'c2.id',
 			c2.name as 'c2.name',
 			c2.description as 'c2.description',
 			c2.category_id as 'c2.category_id'

		from propertyval a 
 			left outer join propertyref c2 on c2.id =  a.propertyref_id 
 			left outer join marker b1 on b1.id =  a.marker_id 

		;    
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC);           
            return $results;

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            $this->count = 0;
            return false;
        }
    }

    /*
        Update propertyval 
        
        parameters : $id (reccord id) and fields list values.
        return : True if no error.
    */
    function update( $id, $marker_id,$propertyref_id,$value,$datecreated ) {

        $this->lasterror="";        

		if ($marker_id == '') $marker_id = null;
		if ($propertyref_id == '') $propertyref_id = null;
		if ($value == '') $value = null;
		if ($datecreated == '') $datecreated = null;
		

        if ($this->checkValue( array( 
		$marker_id => '',
		$propertyref_id => '',
		$value => '',
		$datecreated => ''

) ) == false )
        {
            $this->lasterror="Check fields false";        
            return false;
        }

        $sqlSt="
		update propertyval set
			marker_id = :marker_id,
			propertyref_id = :propertyref_id,
			value = :value,
			datecreated = :datecreated

		where id = :id; 
            ";

        try {        
            
            $stmt = $this->db->prepare($sqlSt);
            
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);

			$stmt->bindParam(':marker_id', $marker_id, PDO::PARAM_INT); 
			$stmt->bindParam(':propertyref_id', $propertyref_id, PDO::PARAM_INT); 
			$stmt->bindParam(':value', $value, PDO::PARAM_STR); 
			$stmt->bindParam(':datecreated', $datecreated, PDO::PARAM_STR); 
 
            $stmt->execute();             
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql update '.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>marker_id</b> = [$marker_id],<b>propertyref_id</b> = [$propertyref_id],<b>value</b> = [$value],<b>datecreated</b> = [$datecreated]";
				
				return false;
			}   
			         
            return true;

        } catch (Exception $e) {            
            $this->lasterror = $e->getMessage().' (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>marker_id</b> = [$marker_id],<b>propertyref_id</b> = [$propertyref_id],<b>value</b> = [$value],<b>datecreated</b> = [$datecreated]";  
            return false;
        }

    }

    /*
        Insert new reccord into propertyval.
        
        Parameters : fields values list.
        return : True if no error.
    */
    function insert( $marker_id,$propertyref_id,$value,$datecreated ) {

        $this->lasterror="";
        $this->lastInsertId = -1;

		if ($marker_id == '') $marker_id = null;
		if ($propertyref_id == '') $propertyref_id = null;
		if ($value == '') $value = null;
		if ($datecreated == '') $datecreated = null;
		
		
        if ($this->checkValue( array( 
		$marker_id => '',
		$propertyref_id => '',
		$value => '',
		$datecreated => ''

) ) == false )
        {
            $this->lasterror="Check fields error";        
            return false;
        }

        $sqlSt="
		insert into propertyval (
		marker_id,
		propertyref_id,
		value,
		datecreated
 
			)
		values (
		:marker_id, 
		:propertyref_id, 
		:value, 
		:datecreated 
 
			);    
            ";

        try {        

            $stmt = $this->db->prepare($sqlSt);               
			
			$stmt->bindParam(':marker_id', $marker_id, PDO::PARAM_INT); 
			$stmt->bindParam(':propertyref_id', $propertyref_id, PDO::PARAM_INT); 
			$stmt->bindParam(':value', $value, PDO::PARAM_STR); 
			$stmt->bindParam(':datecreated', $datecreated, PDO::PARAM_STR); 
 

            $stmt->execute();     
            $this->lastInsertId = $this->db->lastInsertId(); 
            
            if ($stmt->rowCount() == 0) {
				$this->lasterror = 'Erreur Sql insert'.'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>marker_id</b> = [$marker_id],<b>propertyref_id</b> = [$propertyref_id],<b>value</b> = [$value],<b>datecreated</b> = [$datecreated]";
				return false;
			}              
                   
            return true;

        } catch (Exception $e) {
             $this->lasterror = $e->getMessage().'  (<b>Id</b> = '.$id.'). <b>Sqlst</b> = ['.$sqlSt.'] values : '."<b>marker_id</b> = [$marker_id],<b>propertyref_id</b> = [$propertyref_id],<b>value</b> = [$value],<b>datecreated</b> = [$datecreated]";
            return false;
        }
    }

    /*
        Delete one reccord from propertyval.
        
        parameters : $id (reccord id).
        return : True if no error.
    */
    function delete($id) {

        $this->lasterror="";

        $sqlSt="
		delete from propertyval             
		where id = :id; 
            ";   
     
        try {        

            $stmt = $this->db->prepare($sqlSt);  
            $stmt->bindParam(':id', $id, PDO::PARAM_INT);
            $stmt->execute();        
            
			if ($stmt->rowCount() == 0) {
				$this->lasterror = "Erreur Sql delete ".' (<b>Id</b> ='.$id.')';       
				return false;
			}              

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage().' (<b>Id</b>='.$id.')';
            return false;
        }
        return true;
    }

    /*
        Get id min, id max, record count from propertyval  
        
        parameters : None
        return : count value or False in case of error
    */
	function getCount() {
		
        $this->lasterror="";

        $sqlSt="
		select min(id) as minid, max(id) as maxid, count(*) as count 
		from propertyval 
                ";

        try {        

            $stmt = $this->db->prepare($sqlSt);        
            $stmt->execute(); 
            $results = $stmt->fetchAll(PDO::FETCH_ASSOC); 
            return $results[0];          

        } catch (Exception $e) {
            $this->lasterror = $e->getMessage();
            return array( 'minid' => -1, 'maxid' => -1, 'count' => -1);
        }
        		
		//return $this->count;
		return $results[0];
	}




	public $colmeta = [
	'a.id' => array (
		'metades' => "Identifiant",
		'sortformat' => "{{sortformat}}",
		'listlen' => 4,
		'listlib' => 'Id',
		'listlink' => 'on',
		'selectlen' => 4,
		'selectlib' => 'Id',
		'selectlink' => '',
		'selectdisplay' => '',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => '', 
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 

		),
	'a.marker_id' => array ( 
		'metades' => "Identifiant du marker",
		'sortformat' => "", 
		'listlen' => 4,
		'listlib' => 'Marker' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'Marker' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.propertyref_id' => array ( 
		'metades' => "Propriété",
		'sortformat' => "", 
		'listlen' => 4,
		'listlib' => 'Propriété' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'Propriété' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.value' => array ( 
		'metades' => "Valeur de cette propriété",
		'sortformat' => "", 
		'listlen' => 30,
		'listlib' => 'Valeur' , 
		'listlink' => '',
		'selectlen' => 12,
		'selectlib' => 'Valeur' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),
	'a.datecreated' => array ( 
		'metades' => "Date de création ou modification",
		'sortformat' => "date", 
		'listlen' => 8,
		'listlib' => 'Modifié le' , 
		'listlink' => '',
		'selectlen' => 4,
		'selectlib' => 'Modifié le' , 
		'selectlink' => '',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
),

 	'b1.id' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.key' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'key', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'key', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.name' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'name', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'name', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.type' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 3,
		'listlib' => 'type', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'type', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.description' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'description', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'description', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.lat' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'lat', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'lat', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.lon' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'lon', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'lon', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.alt' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'alt', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'alt', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.url' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'url', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'url', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.iconurl' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'iconurl', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'iconurl', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.photoprefix' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'photoprefix', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'photoprefix', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.datemaj' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'datemaj', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'datemaj', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.category_id' => array (  // Marker
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'category_id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'category_id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'b1.datecreated' => array (  // Marker
		'metades' => "",
		'sortformat' => "date",
		'listlen' => 0,
		'listlib' => 'datecreated', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'datecreated', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'c2.id' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'c2.name' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 12,
		'listlib' => 'name', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'name', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'c2.description' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 0,
		'listlib' => 'description', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'description', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
), 
 	'c2.category_id' => array (  // Propertyref
		'metades' => "",
		'sortformat' => "",
		'listlen' => 8,
		'listlib' => 'category_id', 
		'listlink' => '' ,
		'selectlen' => 8,
		'selectlib' => 'category_id', 
		'selectlink' => '{{fkselectlink}}',
		'selectdisplay' => 'text',
		'selectdisplaywidth' => '0', 
		'selectdisplayheight' => '0', 
		'listdisplay' => 'text',
		'listdisplaywidth' => '0', 
		'listdisplayheight' => '0' 
) 
			
	];

    /*
        Get field property value
        
        Parameters : field name and property name.
        return : value of property.
    */

	function getMeta( $fieldkey , $metakey ) {
		return $this->colmeta [$fieldkey][$metakey];
	}

    /*
        Get labels for list table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderList() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['listlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['listlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenList( $field ) {		
		
		return $this->colmeta[$field]['listlen'];
	}

	function getTableFieldLinkList( $field ) {		
		
		return $this->colmeta[$field]['listlink'];
	}

    /*
        Get labels for select table header.
        
        Parameters : None.
        return : String.
    */

	function getTableHeaderSelect() {		
		
		$labelsStr ='';
		
		foreach (  $this->colmeta as $k => $v) {
			if ( $v['selectlen'] > 0 ) 
				$labelsStr = $labelsStr.'<th title="'.$v['metades'].'" >'.$v['selectlib'] .'</th>'."\n";
		}				
		return $labelsStr;
	}
	
	function getTableFieldLenSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlen'];
	}

	function getTableFieldLinkSelect( $field ) {		
		
		return $this->colmeta[$field]['selectlink'];
	}

	
    /*
        Check user input values for propertyval.
        
        Parameters : array of fields with regexp  paterns. ex: array( $nom =>'^(?!\s*$).+' , $icone => '')
        return : False if at last one regexp does not match.
    */
    function checkValue( $arrayCheck ) {

        $indexfield=1;
        foreach ( $arrayCheck as $value => $pattern) {

            if ($pattern=='') continue;

            $pattern='/'.$pattern.'/';

            if (!preg_match($pattern, $value)) {
                $this->lasterror = "Erreur champ #".$indexfield. '. Valeur:['.$value.']';
                return false;                
            }

            $indexfield+=1;

        }

        return true;
    }

} // end class

?>
